%bcond_with xemacs

# Address randomization breaks gcl's memory management scheme
%undefine _hardened_build
# -fstack-protector leads to segfaults because GCL uses its own conflicting
# stack protection scheme.
%undefine _ssp_cflags

# The package notes feature leads to failed builds for everything that depends
# on GCL.  Turn it off until somebody figures out how to make it work without
# polluting linker flags.
%undefine _package_note_file

# GCL wants to read uncompressed info files.  Turn off automatic compression.
%global __brp_compress %{_bindir}/true

# FASL file loads fail with an unexpected EOF error without this.  I do not yet
# know why.
%global __brp_strip_lto %{_bindir}/true

# Use of LTO leads to strange segfaults, reason as yet unknown.
%global _lto_cflags %{nil}

Name:           gcl
Version:        2.6.14
Release:        %mkrel 2
Summary:        GNU Common Lisp

Group:          Development/Other
# LGPL-2.0-or-later:
# - cmpnew
# - gcl-tk/guis.c
# - gcl-tk/tkinfo.lsp
# - gcl-tk/tkl.lsp
# - gmp4 (not used)
# - h
# - lsp (except as noted below)
# - o (except as noted below)
# GPL-1.0-or-later:
# - o/firstfile.c
# - o/lastfile.c
# - o/ntheap.h (not used on Linux)
# - o/unexec*
# - xgcl-2
# MIT-Modern-Variant:
# - gcl-tk/tkAppInit.c
# - gcl-tk/tkMain.c
# - gcl-tk/tkXAppInit.c
# - gcl-tk/tkXshell.c
# LOOP:
# - lsp/gcl_loop.lsp
License:        LGPL-2.0-or-later AND GPL-1.0-or-later AND MIT-Modern-Variant AND LOOP
URL:            https://www.gnu.org/software/gcl/
Source0:        https://ftp.gnu.org/gnu/%{name}/%{name}-%{version}.tar.gz
Source1:        https://www.gnu.org/software/gcl/%{name}-%{version}.tar.gz.sig
Source2:        https://ftp.gnu.org/gnu/gnu-keyring.gpg
Source3:        gcl.el

# Upstream builds point releases for Debian, and uploads the patches directly
# to the Debian Patch Tracker, but does not spin new tarballs.  These are the
# upstream patches from https://sources.debian.org/patches/gcl/.
%global patchdir %{version}-11
Patch0:         https://sources.debian.org/data/main/g/gcl/%{patchdir}/debian/patches/Version_2_6_15pre1
Patch1:         https://sources.debian.org/data/main/g/gcl/%{patchdir}/debian/patches/Version_2_6_15pre2
Patch2:         https://sources.debian.org/data/main/g/gcl/%{patchdir}/debian/patches/Version_2_6_15pre3
Patch3:         https://sources.debian.org/data/main/g/gcl/%{patchdir}/debian/patches/Version_2_6_15pre4
Patch4:         https://sources.debian.org/data/main/g/gcl/%{patchdir}/debian/patches/Version_2_6_15pre5
Patch5:         https://sources.debian.org/data/main/g/gcl/%{patchdir}/debian/patches/Version_2_6_15pre6
Patch6:         https://sources.debian.org/data/main/g/gcl/%{patchdir}/debian/patches/Version_2_6_15pre7
Patch7:         https://sources.debian.org/data/main/g/gcl/%{patchdir}/debian/patches/Version_2_6_15pre8
Patch8:         https://sources.debian.org/data/main/g/gcl/%{patchdir}/debian/patches/Version_2_6_15pre9
Patch9:         https://sources.debian.org/data/main/g/gcl/%{patchdir}/debian/patches/Version_2_6_15pre10

### Fedora patches

# This patch was last sent upstream on 29 Dec 2008.  It updates one source file
# from LaTeX 2.09 to LaTeX 2e, thereby eliminating LaTeX warnings about running
# in compatibility mode.
Patch500:       %{name}-2.6.11-latex.patch
# This patch was last sent upstream on 29 Dec 2008.  It adapts to texinfo 5.0.
Patch501:       %{name}-2.6.11-texinfo.patch
# This patch was last sent upstream on 29 Dec 2008.  It fixes a large number of
# compile- and run-time problems with the Emacs interface code.
Patch502:       %{name}-2.6.11-elisp.patch
# This is a Fedora-specific patch.  Do not delete C files produced from D files
# so they can be pulled into the debuginfo package.
Patch503:       %{name}-2.6.11-debuginfo.patch
# This patch was last sent upstream on 13 Oct 2009.  It fixes two bugs in the
# reading of PLT information.
Patch504:       %{name}-2.6.11-plt.patch
# This patch was last sent upstream on 13 Oct 2009.  It fixes several malformed
# function prototypes involving an ellipsis.
Patch505:       %{name}-2.6.11-ellipsis.patch
# Turn address randomization off early.  GCL is linked with libtirpc, which is
# linked with libselinux, which has a static initializer that calls malloc()
# and free() on systems that do not have /sys/fs/selinux or /selinux mounted,
# or have them mounted read-only.
Patch506:       %{name}-2.6.12-libselinux.patch
# This patch was last sent upstream on 29 Dec 2008.  It updates the autoconf
# and libtool files to newer versions.  By itself, this patch accomplishes
# little of interest.  However, some of the later patches change configure.in.
# Without this patch, autoconf appears to run successfully, but generates a
# configure script that contains invalid shell script syntax.
Patch507:       %{name}-2.6.12-infrastructure.patch
# This patch was last sent upstream on 29 Dec 2008.  It rationalizes the
# handling of system extensions.  For example, on glibc-based systems, some
# functionality is available only when _GNU_SOURCE is defined.
Patch508:       %{name}-2.6.12-extension.patch

BuildRequires:  binutils-devel
BuildRequires:  bzip2
BuildRequires:  gcc
BuildRequires:  gmp-devel
BuildRequires:  gnupg2
BuildRequires:  make
BuildRequires:  pkgconfig(libtirpc)
BuildRequires:  pkgconfig(readline)
BuildRequires:  pkgconfig(sm)
BuildRequires:  pkgconfig(tcl)
BuildRequires:  pkgconfig(tk)
BuildRequires:  pkgconfig(xext)
BuildRequires:  pkgconfig(xaw7)
BuildRequires:  texlive
BuildRequires:  texlive-texmf
BuildRequires:  texinfo
BuildRequires:  emacs-bin
BuildRequires:  emacs-el
%if %{with xemacs}
BuildRequires:  xemacs
BuildRequires:  xemacs-devel
%endif
BuildRequires:  x11-server-common

%description
GCL is a Common Lisp currently compliant with the ANSI standard.  Lisp
compilation produces native code through the intermediary of the
system's C compiler, from which GCL derives efficient performance and
easy portability.  The GUI currently uses TCL/Tk.


%package emacs
License:        GPL-1.0-or-later
Group:          Development/Other
Summary:        Emacs mode for interacting with GCL
Requires:       %{name} = %{version}-%{release}
Requires:       emacs >= %{_emacs_version}
BuildArch:      noarch

%description emacs
Emacs mode for interacting with GCL

%if %{with xemacs}
%package xemacs
Summary:        XEmacs mode for interacting with GCL
Requires:       %{name} = %{version}-%{release}
Requires:       xemacs >= %{_xemacs_version}
Requires:       xemacs-packages-extra
BuildArch:      noarch

%description xemacs
XEmacs mode for interacting with GCL
%endif


%prep
%{gpgverify} --data=%{SOURCE0} --signature=%{SOURCE1} --keyring=%{SOURCE2}

%autosetup -p1

# The binary MUST be run with address randomization off.  The main() function
# has code to accomplish that, but it does not run early enough.  Ensure that
# randomization is off before GCL even starts.
# (tv) disabled: we don't enable selinux
#sed -i 's,echo exec,& %%{_bindir}/setarch -R,' makefile

# Ensure the frame pointer doesn't get added back
sed -i 's/"-fomit-frame-pointer"/""/' configure

# Fix a path in the launch script
sed -i 's|/usr/lib/tk|%{_datadir}/tk|' debian/gcl.sh

# Silence warnings about the obsolescence of egrep and fgrep
sed -i 's/egrep/grep -E/' o/egrep-def
sed -i 's/fgrep/grep -F/' configure.in configure mp/makefile o/unexec.c \
    o/unexec-19.29.c xbin/notify

# Get a version of texinfo.tex that works with the installed version of texinfo
cp -p %{_datadir}/texmf-dist/tex/texinfo/texinfo.tex info

# The archive is so full of spurious executable bits that we just remove them
# all here, then add back the ones that should exist
find . -type f -perm /0111 | xargs chmod a-x
chmod a+x add-defs add-defs1 config.guess config.sub configure install.sh
chmod a+x bin/info bin/info1 gcl-tk/gcltksrv.in gcl-tk/ngcltksrv mp/gcclab
chmod a+x o/egrep-def utils/replace xbin/*

%build
export CFLAGS=$(echo %{optflags} | sed "s/-fstack-clash-protection -fcf-protection=full//")

%configure --enable-readline --enable-ansi --enable-dynsysgmp --enable-xgcl \
  --enable-tclconfig=%{_libdir} --enable-tkconfig=%{_libdir}
# FIXME: %%{?_smp_mflags} breaks the build
make

# Build gcl.info, which is needed for DESCRIBE to work properly
rm info/gcl.info
make -C info gcl.info

# dwdoc needs two extra LaTeX runs to resolve references
cd xgcl-2
pdflatex dwdoc.tex
pdflatex dwdoc.tex
cd -

%install
%make_install

# Get rid of the parts that we don't want
rm -f $RPM_BUILD_ROOT%{_infodir}/dir
rm -rf $RPM_BUILD_ROOT%{_datadir}/doc
rm -rf $RPM_BUILD_ROOT%{_datadir}/emacs
rm -rf $RPM_BUILD_ROOT%{_prefix}/lib/gcl-*/info

# Install the man page
mkdir -p $RPM_BUILD_ROOT%{_mandir}/man1
cp -pf man/man1/* $RPM_BUILD_ROOT%{_mandir}/man1

# Install the HTML documentation
mkdir -p html
cp -pfr info/gcl-si info/gcl-tk html

# Install and compile the Emacs code
mkdir -p $RPM_BUILD_ROOT%{_emacs_sitelispdir}/gcl
cp -pfr elisp/* $RPM_BUILD_ROOT%{_emacs_sitelispdir}/gcl
rm -f $RPM_BUILD_ROOT%{_emacs_sitelispdir}/gcl/makefile
rm -f $RPM_BUILD_ROOT%{_emacs_sitelispdir}/gcl/readme
mkdir -p $RPM_BUILD_ROOT%{_emacs_sitestartdir}
sed -e "s|%LISP_DIR%|%{_emacs_sitelispdir}|" %{SOURCE3} > $RPM_BUILD_ROOT%{_emacs_sitestartdir}/gcl.el
pushd $RPM_BUILD_ROOT%{_emacs_sitelispdir}/gcl
%{_emacs_bytecompile} *.el
popd

%if %{with xemacs}
# Install and compile the XEmacs code
mkdir -p $RPM_BUILD_ROOT%{_xemacs_sitelispdir}/gcl
cp -fr elisp/* $RPM_BUILD_ROOT%{_xemacs_sitelispdir}/gcl
rm -f $RPM_BUILD_ROOT%{_xemacs_sitelispdir}/gcl/makefile
rm -f $RPM_BUILD_ROOT%{_xemacs_sitelispdir}/gcl/readme
mkdir -p $RPM_BUILD_ROOT%{_xemacs_sitestartdir}
sed -e "s|%LISP_DIR%|%{_xemacs_sitelispdir}|" %{SOURCE1} > $RPM_BUILD_ROOT%{_xemacs_sitestartdir}/gcl.el
pushd $RPM_BUILD_ROOT%{_xemacs_sitelispdir}/gcl
%{_xemacs_bytecompile} *.el
popd
%endif

# Help the debuginfo generator
ln -s ../h/cmpinclude.h cmpnew/cmpinclude.h
ln -s ../h/cmpinclude.h lsp/cmpinclude.h
ln -s ../h/cmpinclude.h xgcl-2/cmpinclude.h

# The image has garbage strings containing RPM_BUILD_ROOT
export QA_SKIP_BUILD_ROOT=1



%files
%{_bindir}/gcl
%{_prefix}/lib/gcl*
%{_infodir}/*
%{_mandir}/man*/*
%doc readme readme.xgcl RELEASE* ChangeLog* faq doc
%doc gcl*.jpg gcl.ico gcl.png
%doc html/gcl-si html/gcl-tk
%license COPYING*

%files emacs
%doc elisp/readme
%{_emacs_sitelispdir}/gcl/
%{_emacs_sitestartdir}/*

%if %{with xemacs}
%files xemacs
%doc elisp/readme
%{_xemacs_sitelispdir}/gcl/
%{_xemacs_sitestartdir}/*
%endif


%changelog
* Mon Oct 21 2024 ns80 <ns80> 2.6.14-2.mga10
+ Revision: 2109215
- try building without -fstack-protector-strong -fstack-clash-protection -fcf-protection=full
- revert modification: changing _fortify_level does not fix crash
- try without the fortify level to fix build crash
- try reducing the fortify level to fix build crash
- sync with Fedora to try to fix build

* Sun Nov 05 2023 daviddavid <daviddavid> 2.6.14-1.mga10
+ Revision: 2004397
- update to 2.6.14

* Sun Oct 30 2022 ghibo <ghibo> 2.6.13-0.124.1.mga9
+ Revision: 1900299
- Sync with FC version 2.6.13pre119.
- Update to prerelease 2.6.13pre124.
- Refresh patchset with FC.
- Remove unused patches.
- Pass doc twice through latex.

* Tue Feb 23 2021 wally <wally> 2.6.13-0.99.1.mga9
+ Revision: 1675686
- update to 2.6.13pre99

* Thu Jan 07 2021 tv <tv> 2.6.13-0.95.4.mga8
+ Revision: 1669439
- Update to 2.6.13pre95
- disable selinux patch as we don't have it
- try harder to pass on aarch64
- no more need to set -fcommon, it's set by upstream
- drop old obsoletes tag
- fix mismerge in description
- use %%license
- Update to 2.6.13pre93
+ joequant <joequant>
- fix aarch64 build
- fix gcc10 build
- fix gcc10 build
+ ovitters <ovitters>
- remove install-info scriptlets, rely on filetrigger instead (mga#27107)
+ umeabot <umeabot>
- Mageia 8 Mass Rebuild
+ wally <wally>
- replace deprecated %%configure2_5x

* Sat Jan 04 2020 luigiwalser <luigiwalser> 2.6.12-8.mga8
+ Revision: 1475947
- rebuild for readline

* Sun Oct 14 2018 wally <wally> 2.6.12-7.mga7
+ Revision: 1320259
- sync patches and also a bit of .spec with Fedora
+ umeabot <umeabot>
- Mageia 7 Mass Rebuild
+ kekepower <kekepower>
- Use make_install

* Sun Jan 21 2018 wally <wally> 2.6.12-5.mga7
+ Revision: 1195563
- add patches from Fedora and Debian to fix build issues in other pkgs like maxima

* Tue Jan 02 2018 wally <wally> 2.6.12-4.mga7
+ Revision: 1189430
- rebuild for new readline

* Fri Feb 12 2016 marja <marja> 2.6.12-3.mga6
+ Revision: 959176
- Start to obsolete cgcl-xemacs and gcl-xemacs-el < 2.6.12-3

* Fri Nov 27 2015 pterjan <pterjan> 2.6.12-2.mga6
+ Revision: 906409
- Do not use libbfd, the code for it is in a really broken state
- Build with gnu89 semantics to fix most problems
+ tv <tv>
- build fix from FC
- new snapshot
+ daviddavid <daviddavid>
- apply patch100 to fix build with tcl 8.6
- do not apply patch18 as it is for tcl 8.5
- rebuild for new tcl/tk 8.6.4

* Wed Oct 15 2014 umeabot <umeabot> 2.6.8.20130823-5.mga5
+ Revision: 742900
- Second Mageia 5 Mass Rebuild

* Mon Oct 13 2014 umeabot <umeabot> 2.6.8.20130823-4.mga5
+ Revision: 738482
- Mageia 5 Mass Rebuild

* Mon Jan 06 2014 danf <danf> 2.6.8.20130823-3.mga4
+ Revision: 565026
- Remove -fPIC from CFLAGS as it is incompatible with gcl
+ umeabot <umeabot>
- Mageia 4 Mass Rebuild
- Mass Rebuild - https://wiki.mageia.org/en/Feature:Mageia3MassRebuild
+ luigiwalser <luigiwalser>
- BR texlive-texmf
+ fwang <fwang>
- omi-frame-pointer make it SGC
- 2.6.8 final
+ kamil <kamil>
- sync sources with Fedora
- rebuild for Mga3 Beta1

* Mon Mar 05 2012 kamil <kamil> 2.6.8.20120109cvs-2.mga2
+ Revision: 218547
- rebuild against new gcc-4.6.3

* Mon Feb 06 2012 kamil <kamil> 2.6.8.20120109cvs-1.mga2
+ Revision: 205242
- sync source and patches with Fedora (20120109svn)
- build against emacs 23.3
- build against xemacs 21.4.22

* Wed Jul 06 2011 fwang <fwang> 2.6.8.20101115cvs-4.mga2
+ Revision: 119210
- rebuild for new tcl

* Wed May 11 2011 pterjan <pterjan> 2.6.8.20101115cvs-3.mga1
+ Revision: 97333
- Don't require a version of xemacs that we don't have yet

* Fri Apr 29 2011 dmorgan <dmorgan> 2.6.8.20101115cvs-2.mga1
+ Revision: 93455
- Minor layout change

* Thu Apr 14 2011 tv <tv> 2.6.8.20101115cvs-1.mga1
+ Revision: 85467
- fix texlive BR
- imported package gcl


* Sun Jan 30 2011 Paulo Andrade <pcpa@mandriva.com.br> 2.6.8.20101115cvs-1mdv2011.0
+ Revision: 634037
- Update to newer 2.6.8 snapshot

* Sat Aug 07 2010 Paulo Andrade <pcpa@mandriva.com.br> 2.6.8.20100201cvs-1mdv2011.0
+ Revision: 567209
- Update to a newer CVS snapshot

* Wed Feb 10 2010 Funda Wang <fwang@mandriva.org> 2.6.8.20090701cvs-3mdv2010.1
+ Revision: 503674
- fix build with gmp 5.0
- rebuild for new gmp

* Fri Nov 27 2009 Paulo Andrade <pcpa@mandriva.com.br> 2.6.8.20090701cvs-2mdv2010.1
+ Revision: 470707
- Correct problems with newer binutils causing gcl to miss read the linker map

* Wed Nov 25 2009 Paulo Andrade <pcpa@mandriva.com.br> 2.6.8.20090701cvs-1mdv2010.1
+ Revision: 470103
- Update to newer gc-2.6.8 pre snapshot.

* Wed Nov 25 2009 Paulo Andrade <pcpa@mandriva.com.br> 2.6.8.20090303cvs-5mdv2010.1
+ Revision: 469867
- Build correction for newer binutils.
- Add switch to statically link to libbfd.
- Rebuild due to newer binutils.

  + Per Øyvind Karlsen <peroyvind@mandriva.org>
    - disable -fstack-protector by overriding the %%_ssp_cflags macro that sets it

* Tue Nov 10 2009 Paulo Andrade <pcpa@mandriva.com.br> 2.6.8.20090303cvs-4mdv2010.1
+ Revision: 464339
- Correct CFLAGS as GCL has its own stack protector schema

* Sat Nov 07 2009 Frederik Himpe <fhimpe@mandriva.org> 2.6.8.20090303cvs-3mdv2010.1
+ Revision: 462595
- Don't mark subpackages as Noarch as our bs does not deal with this yet

* Sat Nov 07 2009 Frederik Himpe <fhimpe@mandriva.org> 2.6.8.20090303cvs-2mdv2010.1
+ Revision: 462330
- Bump release for rebuild for missing package

* Tue Aug 18 2009 Paulo Andrade <pcpa@mandriva.com.br> 2.6.8.20090303cvs-1mdv2010.0
+ Revision: 417506
- make selinux support optional, and default to disabled
- Update to 2.6.8, cvs 20090303.

* Sat Dec 06 2008 Adam Williamson <awilliamson@mandriva.org> 2.6.7-10mdv2009.1
+ Revision: 311069
- rebuild for new tcl

* Sat Aug 30 2008 Frederik Himpe <fhimpe@mandriva.org> 2.6.7-9mdv2009.0
+ Revision: 277528
- Make it build again:
  * add huge Debian diff, fixing many bugs (notably gnu-hash style
    support) and adding xgcl-2
  * Remove gcl-2.6.7-conf.patch, already part of Debian patch
  * Add some configure options from Gentoo

  + Thierry Vignaud <tv@mandriva.org>
    - rebuild

* Mon Jan 07 2008 Adam Williamson <awilliamson@mandriva.org> 2.6.7-6mdv2008.1
+ Revision: 146200
- rebuild for new tcl/tk and new era
- don't run autoconf any more. this makes most of the patch a no-op, but most of it is not actually useful. replace the emacs part of it with a configure option
- clean spec
- new license policy, correct license (compiled app is GPL, not LGPL)

  + Olivier Blin <oblin@mandriva.com>
    - restore BuildRoot

  + Thierry Vignaud <tv@mandriva.org>
    - kill re-definition of %%buildroot on Pixel's request


* Thu Mar 15 2007 Giuseppe Ghibò <ghibo@mandriva.com> 2.6.7-5mdv2007.1
+ Revision: 144128
- Rebuilt because corrupted RPM (bug #29516).

* Sun Feb 18 2007 Giuseppe Ghibò <ghibo@mandriva.com> 2.6.7-4mdv2007.1
+ Revision: 122448
- BuildRequires: automake1.7.
- Added Patch0 for configuring again.
- Import gcl

* Mon Jan 02 2006 Oden Eriksson <oeriksson@mandriva.com> 2.6.7-3mdk
- rebuilt against soname aware deps (tcl/tk)
- fix deps

* Sat Oct 08 2005 Nicolas Lécureuil <neoclust@mandriva.org> 2.6.7-2mdk
- Fix BuildRequires
- %%mkrel

* Wed Aug 31 2005 Giuseppe Ghibò <ghibo@mandriva.com> 2.6.7-1mdk
- Release 2.6.7.

* Thu Aug 04 2005 Giuseppe Ghibò <ghibo@mandriva.com> 2.6.7-0.cvs20050803.1mdk
- Release 2.6.7pre (current CVS).

* Fri Apr 29 2005 Giuseppe Ghibò <ghibo@mandrakesoft.com> 2.6.6-4mdk
- Added emacs to BuildRequires.

* Mon Apr 18 2005 Giuseppe Ghibò <ghibo@mandrakesoft.com> 2.6.6-3mdk
- Added binutils-devel to Requires.
- Added fixes to SPEC file for building under X86-64.

* Wed Feb 16 2005 Giuseppe Ghibò <ghibo@mandrakesoft.com> 2.6.6-2mdk
- Enabled ansi.

* Wed Feb 16 2005 Giuseppe Ghibò <ghibo@mandrakesoft.com> 2.6.6-1mdk
- Release: 2.6.6.

* Tue Oct 05 2004 Giuseppe Ghibò <ghibo@mandrakesoft.com> 2.6.5-2mdk
- Fix permissions.

* Tue Oct 05 2004 Giuseppe Ghibò <ghibo@mandrakesoft.com> 2.6.5-1mdk
- Release 2.6.5.

* Tue Jul 20 2004 Giuseppe Ghibò <ghibo@mandrakesoft.com> 2.6.3-1mdk
- Release 2.6.3.

